/**=================================================================================================
Hochschule Luzern - Technik & Architektur                                   Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                     http://hslu.ximit.ch
----------------------------------------------------------------------------------------------------
Project     : Adressverwaltung
Name        : address.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-19v1.0
Description : Adressverwaltung
==================================================================================================*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "address.h"
#include "addressFileIO.h"

// Global VAR: Addresslist
addressList_t* addrList = NULL;

err_t initialize(void)
{
    //create a new List
    addrList = (addressList_t*) malloc(sizeof(addressList_t));
    if (addrList) { // allocate Memory successfully?
        addrList->addr = NULL;
        addrList->size = 0;
        return SUCCESS;
    } else {
        return NOMEMORY;
    };
}

void listAddress(void (*showAddrEntryFN)(int, address_t*))
{
    int i; // tmp counter

    for (i=0; i<addrList->size; i++) {
        showAddrEntryFN(i+1, &(addrList->addr[i])); // Use CallBack Function for Output
    }
}

err_t addAddressValues(char* lastname, char* firstname, char* street, int zip, char* city)
{
    address_p newAddr; // Address Object to fill and add
    
    strcpy(newAddr.firstname, firstname);
    strcpy(newAddr.lastname, lastname);
    strcpy(newAddr.street, street);
    newAddr.zip = zip;
    strcpy(newAddr.city, city);
    return addAddress(newAddr);
}

err_t addAddress(address_p newAddress)
{
    int count; // Address Counter
    
    count = addrList->size;
    if (count == 0) {
        // alloc new memory
        addrList->addr = (address_t*) malloc(sizeof(address_t));
    } else {
        // realloc memory
        addrList->addr = (address_t*) realloc(addrList->addr, sizeof(address_t) * (count + 1));
    }
    if (addrList->addr) {
        // Add new Node
        strcpy(addrList->addr[count].firstname, newAddress.firstname);
        strcpy(addrList->addr[count].lastname, newAddress.lastname);
        strcpy(addrList->addr[count].street, newAddress.street);
        addrList->addr[count].zip = newAddress.zip;
        strcpy(addrList->addr[count].city, newAddress.city);
        addrList->addr[count].index = count;
        addrList->size = ++count;
        return SUCCESS;
    } else {
        return NOMEMORY;
    }
}

err_t clearAddress(void)
{
    // release memory
    free(addrList);
    // alloc new memory
    return initialize();
}

err_t saveAddrFile(char* path, int* num)
{
    return writeFile(path, addrList, formatCSVTab, num);
}

err_t readAddrFile(char* path, int* num)
{
    return readFile(path, addrList, lineParser, num);
}

err_t sortAddress(sortmode mode)
{
    int i; // temp counter, used for stable sort
    
    // qsort(Beginning address of array, Number of elements, Size of each element, Pointer to compare function
    switch (mode) {
        case NAME:
            qsort(addrList->addr,addrList->size, sizeof(address_t), (compfn)compareByName);
            break;
        case STREET:
            qsort(addrList->addr,addrList->size, sizeof(address_t), (compfn)compareByStreet);
            break;
        case ZIP:
            qsort(addrList->addr,addrList->size, sizeof(address_t), (compfn)compareByZip);
            break;
        case CITY:
            qsort(addrList->addr,addrList->size, sizeof(address_t), (compfn)compareByCity);
            break;
    }
    // update index - used for stable sort
    for (i=0; i<addrList->size; i++) {
        addrList->addr[i].index = i;
    }
    return SUCCESS;
}

int compareByName(const void *a, const void *b)
{
    int res;                           // errorcode handler
    char name1[2*MAXSTRLEN + 1];       // Buffer for address 1 Name
    char name2[2*MAXSTRLEN + 1];       // Buffer for address 2 Name
    address_t *addr1 = (address_t*) a; // address 1, cast param
    address_t *addr2 = (address_t*) b; // address 2, cast param

    // Copy Lastname and Firstname into one String
    strcpy(name1, addr1->lastname);
    strcat(name1, addr1->firstname);
    strcpy(name2, addr2->lastname);
    strcat(name2, addr2->firstname);
   
    // Compare String Value 'Lastname Firstname'
    //res = strcasecmp(name1, name2); // Compare String (ignore case)
	res = strcmp(name1, name2); // Compare String (ignore case)
    if (res == 0) { // stable sort
        res = addr1->index - addr2->index;
    }
    return res;
}

int compareByStreet(const void *a, const void *b)
{
    int res;                           // errorcode handler
    address_t *addr1 = (address_t*) a; // address 1, cast param
    address_t *addr2 = (address_t*) b; // address 2, cast param
    
    // Compare String Value 'Street'
    //res = strcasecmp(addr1->street, addr2->street); // ignore case
	res = strcmp(addr1->street, addr2->street); // ignore case
    if (res == 0) { // stable sort
        res = addr1->index - addr2->index;
    }
    return res;
}

int compareByZip(const void *a, const void *b)
{
    int res;                           // errorcode handler
    address_t *addr1 = (address_t*) a; // address 1, cast param
    address_t *addr2 = (address_t*) b; // address 2, cast param
    
    // Compare Int value 'Zip'
    res = addr1->zip - addr2->zip;
    if (res == 0) { // stable sort
        res = addr1->index - addr2->index;
    }
    return res;
}

int compareByCity(const void *a, const void *b)
{
    int res;                           // errorcode handler
    address_t *addr1 = (address_t*) a; // address 1, cast param
    address_t *addr2 = (address_t*) b; // address 2, cast param
    
    // Compare String Value 'City'
    //res = strcasecmp(addr1->city, addr2->city);
	res = strcmp(addr1->city, addr2->city);
    if (res == 0) { // stable sort
        res = addr1->index - addr2->index;
    }
    return res;
}
